//	3D Grass Shader - 1.0
//------------------------------------
float4x4 WorldViewProj : WorldViewProjection;
float4x4 World : WorldIT;
float4 EyePos : EyePos;
float Time : Time;

//------------------------------------
float4 LightVector <> = {0.0f, 1.0f, 0.0f, 0.0f};
float4 LightColour <> = {1.0f, 1.0f, 1.0f, 0.0f};
float4 LightScatter <> = {0.3f, 0.4f, 0.0f, 0.0f};
float4 LightAmbient <> = {0.25f, 0.25f, 0.25f, 0.0f};

float4 GrassLength <> = {0.004f, 0.004f, 0.004f, 0.0f};
float4 WindRate <> = {0.11f, 0.04f, 0.0f, 0.0f};

//------------------------------------
texture grassMap
<
	string name = "Grass.tga";
>;

texture windMap
<
	string name = "WindMap.tga";
>;

//------------------------------------
technique Grass14
{
	pass p0 
	{		
		VertexShaderConstant[0] = <WorldViewProj>;
		VertexShaderConstant[4] = <World>;
		VertexShaderConstant[11] = <EyePos>;
		VertexShaderConstant[12] = <Time>;
		VertexShaderConstant[13] = <LightVector>;
		VertexShaderConstant[14] = <LightColour>;
		VertexShaderConstant[15] = <LightScatter>;
		VertexShaderConstant[16] = <LightAmbient>;
		VertexShaderConstant[17] = <GrassLength>;
		VertexShaderConstant[18] = <WindRate>;

		VertexShader = asm
		{
			// Initilise Shader
			vs.1.1
			dcl_position	v0
			dcl_normal	v1
			dcl_tangent	v2
			dcl_binormal	v3
			dcl_texcoord	v4
			
			// Define Constants
			def c20, 1.0f, 0.5f, 0.0f, 0.0f
			
			// Transform Position to Clip
			m4x4 oPos, v0, c0
			
			// Move Eye Position to Object Space
			mov r0, c11
			m4x4 r1, r0, c4
			
			// Compute Normalised Eye Vector
			sub r1, r1, v0
			dp3 r1.w, r1, r1
			rsq r1.w, r1.w
			mul r1, r1, r1.w
			
			// Move Eye Vector to Texture Space
			dp3 r0.x, r1, v2
			dp3 r0.y, r1, v3
			
			// Calculate Grass Lengths
			add r1, c17, c17
			add r2, r1, c17
			
			// Output Grass Layer Coordinates
			mov oT0, v4
			mad oT1, r0, c17, v4
			mad oT2, r0, r1, v4
			mad oT3, r0, r2, v4
			
			// Output Wind Texture
			add r0, v4, c12.x
			mul oT4.xy, r0, c18
			
			// Move Light Vector to Object Space
			mov r0, c13
			m4x4 r1, r0, c4
			
			// Normalise Light Vector
			dp3 r1.w, r1, r1
			rsq r1.w, r1.w
			mul r1, r1, r1.w
			
			// N-dot-L
			dp3 r0, r1, v1
			
			// Clamp Light to [0..1]
			max r1, r0, c20.z
			
			// Compute Simple Fresnel
			max r0, r0, -r0
			sub r0, c20.x, r0
			mul r0, r0, c15
			
			// Apply Colour to Light
			mad r1, r1, c14, r0
			add r1, r1, c16
			mul oD0, r1, c20.y
		};
		
		PixelShader = asm
		{
			// Initialise Shader
			ps.1.4
			def c0, 0.01f, 0.02f, 0.03f, 0.04f
			
			// Load Texture Coordinates
			texcrd r1.rgb, t1
			texcrd r2.rgb, t2
			texcrd r3.rgb, t3
			texld r4, t4
			
			// Peturb Texture Based on Wind
			mad r1.r, r4_bias, c0.r, r1
			mad r2.r, r4_bias, c0.g, r2
			mad r3.r, r4_bias, c0.b, r3
			
			// Phase
			phase
			
			// Load 4 Layers of Grass
			texld r0, t0
			texld r1, r1
			texld r2, r2
			texld r3, r3
			
			// Pick from Two Layers
			sub r4, r1.g, r0.g
			cmp r0, r4, r1, r0
			
			// Pick from Three Layers
			sub r4, r2.g, r0.g
			cmp r0, r4, r2, r0
			
			// Pick from Four Layers
			sub r4, r3.g, r0.g
			cmp r0, r4, r3, r0
			
			// Apply Lighting
			mul r0, r0, v0_x2
		};

		Texture[0] = <grassMap>;
		MinFilter[0] = Linear;
		MagFilter[0] = Linear;
		MipFilter[0] = Linear;

		Texture[1] = <grassMap>;
		MinFilter[1] = Linear;
		MagFilter[1] = Linear;
		MipFilter[1] = Linear;

		Texture[2] = <grassMap>;
		MinFilter[2] = Linear;
		MagFilter[2] = Linear;
		MipFilter[2] = Linear;

		Texture[3] = <grassMap>;
		MinFilter[3] = Linear;
		MagFilter[3] = Linear;
		MipFilter[3] = Linear;

		Texture[4] = <windMap>;
		MinFilter[4] = Linear;
		MagFilter[4] = Linear;
		MipFilter[4] = Linear;
	}
}